@@ -29,15 +29,29 @@ num_benchmark_iteration = 30
 # print info for debugging purpose
 logging_level = logging.INFO
 
-# benchmark accuracy mode
-eval_accuracy = False
-
 # print code line info for debug use
 print_code_line_info = False
 
 # load transformers class def by a cache file instead of on-the-fly catch
 cache_load_transformers = True
 
+# detected device
+device = "cpu_with_amx"
+
+# device compatibility of the code: e.g. ["cpu", "cuda"], ["cuda"]
+list_code_device_compatibility = ["cuda"]
+
+# quantization config for HuggingFace optimum-intel optimizations
+# it is either "" (None) or "xxx" (a string of config path)
+optimum_quant_config = ""
+
+# code domain
+code_domain = ""
+
+# modular design
+use_modular = True
+modular_item = "" # str
+
 def reset_globals():
     global list_code_path
 
@@ -68,6 +82,7 @@ def reset_globals():
 
     global list_calib_dataloader_name
     global list_eval_func_lines
+    global list_eval_func_name
 
     list_code_path = []
     list_code_line_instance = []  # list of CodeLine instances
@@ -97,4 +112,5 @@ def reset_globals():
     list_wrapper_all_function_name = []
 
     list_calib_dataloader_name = []
-    list_eval_func_lines = []
\ No newline at end of file
+    list_eval_func_lines = []
+    list_eval_func_name = []
