@@ -79,8 +79,11 @@ def register_func_wrap_pair():
                     # judge_1: indent is equal to def indent
                     judge_1 = following_indent_level <= def_indent_level
                     # judge_2: not starting with")"
-                    judge_2 = True if (
-                        following_line != "" and following_line[following_indent_level] != ")") else False
+                    try:
+                        judge_2 = True if (
+                            following_line != "" and following_line[following_indent_level] != ")") else False
+                    except:
+                        judge_2 = False
                     # judge_3: is not a comment
                     c1 = False
                     c2 = False
