@@ -17,6 +17,7 @@ import subprocess
 import logging
 import time
 import yaml
+import re
 
 from . import globals
 
@@ -24,12 +25,28 @@ if not os.path.exists("neural_coder_workspace"):
     os.makedirs("neural_coder_workspace")
 
 
+def detect_device_(logger):
+    # device detection
+    logger.info(f"Device detection started ...")
+    from .utils.device import detect_device
+    detect_device()
+    if globals.device == "cpu_with_amx":
+        logger.info(f"Device: CPU with AMX")
+    elif globals.device == "cpu_without_amx":
+        logger.info(f"Device: CPU without AMX")
+    elif globals.device == "intel_gpu":
+        logger.info(f"Device: Intel(R) GPU")
+    elif globals.device == "cuda":
+        logger.info(f"Device: CUDA")
+    elif globals.device == "mutli":
+        logger.info(f"Device: Multi-Device")
+
+
 def enable(
     code,
     features,
     target_batch_size=1,  # effective for feature "pytorch_change_batch_size"
     num_benchmark_iteration=10,  # effective for feature "pytorch_benchmark"
-    eval_accuracy=False,
     generate_patch=True,
     overwrite=False,
     save_patch_path="",
@@ -48,6 +65,10 @@ def enable(
     bench_batch_size=-1,  # only for "self_defined" mode
     test_code_line=False, # print code line info for debug use
     cache_load_transformers=True,
+    optimum_quant_config="", # only for HF optimum optimizations, yaml or hub path
+    use_inc=False,
+    use_modular=False,
+    modular_item="",
 ):
     """enable a feature or a couple of features for the code
 
@@ -81,6 +102,9 @@ def enable(
     logger.addHandler(fh)
     logger.addHandler(ch)
 
+    # device detection
+    detect_device_(logger)
+
     # print key inputs
     logger.info(f"Enabling started ...")
     logger.info(f"code: {code}")
@@ -97,6 +121,8 @@ def enable(
         "pytorch_inc_static_quant_fx",
         "pytorch_inc_static_quant_ipex",
         "pytorch_inc_bf16",
+        "pytorch_inc_huggingface_optimum_static",
+        "pytorch_inc_huggingface_optimum_dynamic",
         "pytorch_ipex_fp32",
         "pytorch_ipex_bf16",
         "pytorch_ipex_int8_static_quant",
@@ -116,6 +142,12 @@ def enable(
         "pytorch_aliblade",
         "tensorflow_amp",
         "keras_amp",
+        "tensorflow_inc",
+        "keras_inc",
+        "onnx_inc_static_quant_qlinear",
+        "onnx_inc_static_quant_qdq",
+        "onnx_inc_dynamic_quant",
+        "inc_auto",
     ]
     '''
 
@@ -146,12 +178,14 @@ def enable(
 
     #### Feature Enabling
 
-    globals.num_benchmark_iteration = str(num_benchmark_iteration + 5) # 5: warmup iteration number
-
-    globals.eval_accuracy = eval_accuracy
+    globals.num_benchmark_iteration = str(num_benchmark_iteration + 10) # 10: warmup iteration number
 
     globals.cache_load_transformers = cache_load_transformers
+    globals.optimum_quant_config = optimum_quant_config
 
+    globals.use_modular = use_modular
+    globals.modular_item = modular_item
+    
     # move "pytorch_benchmark" to the last
     from .utils.common import move_element_to_last
     features = move_element_to_last(features, "pytorch_benchmark")
@@ -162,6 +196,7 @@ def enable(
         "pytorch_cuda_to_cpu",
         "pytorch_lightning_bf16_cpu",
         "tensorflow_mixed_precision",
+        "change_trainer_to_nlptrainer",
     ]
     
     # # features that need creating dummy dataloader (when needed) first
@@ -176,6 +211,10 @@ def enable(
         "pytorch_inc_static_quant_ipex" in features:
         features = ["pytorch_reclaim_inputs"] + features
 
+    # intel_extension_for_transformers
+    if "intel_extension_for_transformers" in features:
+        features = ["change_trainer_to_nlptrainer"] + features
+
     transformed_list_code_path = []
 
     ## Determine Code Domain
@@ -197,6 +236,19 @@ def enable(
     ## Feature Transformation
     for idx_feature, feature in enumerate(features):
 
+        # "inc_auto" auto selection of feature according to fwk
+        if feature == "inc_auto":
+            from .coders.autoinc import domain
+            code_domain = domain.determine_domain(globals.list_code_path[0])
+            if code_domain == "keras_script":
+                feature = "keras_inc"
+            elif code_domain == "tensorflow_keras_model":
+                feature = "tensorflow_inc"
+            elif code_domain == "onnx":
+                feature = "onnx_inc_dynamic_quant"
+            else:
+                feature = "pytorch_inc_dynamic_quant"
+
         # reset globals
         globals.reset_globals()
 
@@ -252,7 +304,34 @@ def enable(
                     "pytorch_inc_dynamic_quant",
                     "pytorch_inc_static_quant_fx",
                     "pytorch_inc_static_quant_ipex",
+                    "pytorch_inc_huggingface_optimum_static",
+                    "pytorch_inc_huggingface_optimum_dynamic",
+                    "onnx_inc_static_quant_qlinear",
+                    "onnx_inc_static_quant_qdq",
+                    "onnx_inc_dynamic_quant",
+                    "intel_extension_for_transformers",
                 ]:
+
+                # determine domain
+                from .coders.autoinc.domain import determine_domain
+                globals.code_domain = determine_domain(globals.list_code_path[0])
+
+                # for transformers code, enable optimum-intel api by default
+                # if specify use_inc, then still use INC API
+                if "transformers" in globals.code_domain and not use_inc:
+                    if "static_quant" in feature:
+                        feature = "pytorch_inc_huggingface_optimum_static"
+                    elif "dynamic_quant" in feature:
+                        feature = "pytorch_inc_huggingface_optimum_dynamic"
+
+                # optimum-intel quantization config for static and dynamic
+                if feature == "pytorch_inc_huggingface_optimum_static":
+                    globals.optimum_quant_config = "quantization/quant_config_static"
+                elif feature == "pytorch_inc_huggingface_optimum_dynamic":
+                    globals.optimum_quant_config = "quantization/quant_config_dynamic"
+                else:
+                    pass
+
                 from .coders.autoinc.autoinc_harness import AutoInc_Harness
                 from .coders.autoinc.calib_dataloader import Calib_Dataloader
                 from .coders.autoinc.eval_func import Eval_Func
@@ -295,6 +374,13 @@ def enable(
                 if "tensorflow_mixed_precision" in features:
                     from .coders.tensorflow.amp import TensorFlowKerasAMP
                     list_transformed_code[i] = TensorFlowKerasAMP(list_transformed_code[i]).transform()
+                if "tensorflow_inc" in features:
+                    from .coders.tensorflow.inc import TensorFlowKerasINC
+                    list_transformed_code[i] = TensorFlowKerasINC(list_transformed_code[i]).transform()
+                # Change Trainer to NLPTrainer (only for intel_extension_for_pytorch)
+                if "change_trainer_to_nlptrainer" in features:
+                    from .coders.pytorch.change_trainer_to_nlptrainer import TrainerToNLPTrainer
+                    list_transformed_code[i] = TrainerToNLPTrainer(list_transformed_code[i]).transform()
 
         logger.info(f"Code transformation for feature: [{feature}] finished.")
 
@@ -348,6 +434,8 @@ def enable(
         whole_patch_user_code = ""
         for path in globals.list_code_path[0:num_user_code_path]:
             path_transformed = path[:-3] + "_nc_enabled.py"
+            if path_transformed[-25:] == "_nc_enabled_nc_enabled.py":
+                continue
             cmd_gen_patch = "diff -up " + path + " " + path_transformed
             sp_gen_patch = subprocess.Popen(
                 cmd_gen_patch, env=os.environ, shell=True, stdout=subprocess.PIPE)  # nosec
@@ -367,7 +455,7 @@ def enable(
             sp_overwrite = subprocess.Popen(
                 "patch -d/ -p0 < " + abs_patch_path, env=os.environ, shell=True, stdout=subprocess.PIPE)  # nosec
             sp_overwrite.wait()
-            os.remove(abs_patch_path)  # remove patch after overwrite
+            # os.remove(abs_patch_path)  # remove patch after overwrite
 
         if patch_imports:
             whole_patch_import_modules = ""
@@ -462,6 +550,9 @@ def bench(
     logger.addHandler(ch)
     logger.addHandler(fh)
 
+    # device detection
+    detect_device_(logger)
+
     # print key inputs
     logger.info(f"Benchmarking started ...")
     logger.info(f"code: {code}")
@@ -561,7 +652,7 @@ def bench(
     count_P90 = 0
     P99 = 0
     count_P99 = 0
-    acc = 0
+    acc_delta = 0
     for line in bench_log:
         if "Neural_Coder_Bench_IPS" in line:
             try:
@@ -592,9 +683,11 @@ def bench(
                 count_P99 += 1
             except ValueError as ve:
                 pass
-        if "Acc@5" in line:
+        if "Accuracy (int8|fp32)" in line:
             try:
-                acc += float(line[line.find(":")+3:])
+                acc_int8 = float(re.search(r"\d+\.\d+", line).group())
+                acc_fp32 = float(re.search(r"(?<=\|)\d+\.\d+", line).group())
+                acc_delta = round((acc_int8 - acc_fp32) / acc_fp32 * 100, 2) # percent of increase/decrease
             except ValueError as ve:
                 pass
 
@@ -632,7 +725,7 @@ def bench(
     logger.info(f"Collected latency_p50 on the code is: [{P50}] (mspi)")
     logger.info(f"Collected latency_p90 on the code is: [{P90}] (mspi)")
     logger.info(f"Collected latency_p99 on the code is: [{P99}] (mspi)")
-    logger.info(f"Collected accuracy on the code is: [{acc}]")
+    logger.info(f"Collected accuracy delta on the code is: [{acc_delta}]")
 
     # unpatch
     if patch_path != "":
@@ -640,7 +733,7 @@ def bench(
             "patch -R -d/ -p0 < " + patch_path, env=os.environ, shell=True, stdout=subprocess.PIPE)  # nosec
         sp_unpatch.wait()
 
-    return [FPS, MSPI, P50, P90, P99, acc], mode, os.path.abspath(ws_path)
+    return [FPS, MSPI, P50, P90, P99, acc_delta], mode, os.path.abspath(ws_path)
 
 
 def superbench(
@@ -654,12 +747,11 @@ def superbench(
     num_benchmark_iteration=5,
     iteration_dynamic_adjust=True,
     logging_level="info",
-    cpu_conversion=True,
     cpu_set_env=True,
     ncore_per_instance=-1,  # only for "self_defined" mode
     ninstances=-1,  # only for "self_defined" mode
     bench_batch_size=-1,  # only for "self_defined" mode
-    eval_accuracy=False,
+    use_inc=False,
     auto_quant=False,
 ):
 
@@ -687,6 +779,9 @@ def superbench(
     logger.addHandler(ch)
     logger.addHandler(fh)
 
+    # device detection
+    detect_device_(logger)
+
     # print key inputs
     if auto_quant:
         logger.info(f"Auto-Quant started ...")
@@ -714,6 +809,10 @@ def superbench(
                 f"You have to specify an entry_code of your code: [{code}]")
             quit()
 
+    # detect device compatibility of entry code
+    from .utils.device import detect_code_device_compatibility
+    detect_code_device_compatibility(entry_code)
+
     if sweep_objective == "feature":
         list_FPS = []
         list_accuracy = []
@@ -797,7 +896,8 @@ def superbench(
                 if "pytorch_inc_dynamic_quant" in features and "pytorch_mixed_precision_cpu" in features:
                     continue
 
-                if cpu_conversion:
+                # device conversion
+                if "cpu" in globals.device and "cpu" not in globals.list_code_device_compatibility:
                     features.append("pytorch_cuda_to_cpu")
 
                 if features[0] == "" and len(features) > 1:
@@ -818,24 +918,8 @@ def superbench(
                     ncore_per_instance=ncore_per_instance,
                     ninstances=ninstances,
                     bench_batch_size=bench_batch_size,
-                    eval_accuracy=False,
+                    use_inc=use_inc,
                 )
-                if eval_accuracy:
-                    bench_acc, bench_mode, bench_ws_path = enable(
-                        code=code,
-                        entry_code=entry_code,
-                        args=args,
-                        features=features,
-                        mode=mode,
-                        run_bench=True,
-                        num_benchmark_iteration=num_benchmark_iteration,
-                        cpu_set_env=cpu_set_env,
-                        ncore_per_instance=ncore_per_instance,
-                        ninstances=ninstances,
-                        bench_batch_size=bench_batch_size,
-                        eval_accuracy=True,
-                    )
-
 
                 if dry_run:
                     t_end = time.time()
@@ -867,86 +951,54 @@ def superbench(
                     elif features == []:
                         features_display = "The Original Model"
 
-                    if not eval_accuracy:
-                        logger.info(
-                            f"Benchmark result (performance) of {features_display}"
-                            f" is {bench_performance[0]} (FPS)")
-                    else:
-                        logger.info(
-                            f"Benchmark result (performance) of {features_display}"
-                            f" is {bench_performance[0]} (FPS)")
-                        logger.info(
-                            f"Benchmark result (accuracy) of {features_display} is {bench_acc[5]}")
+                    logger.info(
+                        f"Benchmark result (performance) of {features_display}"
+                        f" is {bench_performance[0]} (FPS)")
+                    logger.info(
+                        f"Benchmark result (accuracy delta) of {features_display} is {bench_performance[5]} %")
                 else:
-                    if not eval_accuracy:
-                        logger.info(
-                            f"Benchmark result (performance) of optimization set [{features}]"
-                            f" is [{bench_performance[0]}] (FPS)")
-                    else:
-                        logger.info(
-                            f"Benchmark result (performance) of optimization set [{features}]"
-                            f" is [{bench_performance[0]}] (FPS)")
-                        logger.info(
-                            f"Benchmark result (accuracy) of optimization set [{features}] is [{bench_acc[5]}]")
+                    logger.info(
+                        f"Benchmark result (performance) of optimization set [{features}]"
+                        f" is [{bench_performance[0]}] (FPS)")
+                    logger.info(
+                        f"Benchmark result (accuracy delta) of optimization set [{features}]"
+                        f" is [{bench_performance[5]}] %")
 
                 d = {}  # initialize dict
                 d["features"] = features
                 d["FPS"] = bench_performance[0]
-                if eval_accuracy:
-                    d["accuracy"] = bench_acc[5]
+                d["accuracy"] = bench_performance[5]
                 d["mode"] = bench_mode
                 d["workspace_path"] = bench_ws_path
                 result.append(d)
 
                 list_FPS.append(bench_performance[0])
-                if eval_accuracy:
-                    list_accuracy.append(bench_acc[5])
+                list_accuracy.append(bench_performance[5])
                 list_features.append(features)
                 list_mode.append(bench_mode)
                 list_ws_path.append(bench_ws_path)
 
         # print result
-        if not eval_accuracy:
-            print(f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
-            print("{:<20} {:<20} {:<120}".format(
-                'Numactl Mode', 'Performance (FPS)', 'Features Applied'))
+        print(f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
+        print("{:<20} {:<20} {:<20} {:<120}".format(
+            'Numactl Mode', 'Performance (FPS)', 'Accuracy Delta (%)', 'Features Applied'))
 
-            sort_index = sorted(
-                range(len(list_FPS)),
-                key=lambda k: list_FPS[k],
-                reverse=True,
-            )
+        sort_index = sorted(
+            range(len(list_FPS)),
+            key=lambda k: list_FPS[k],
+            reverse=True,
+        )
 
-            for i in sort_index:
-                if list_FPS[i] != 0:
-                    print(
-                        "{:<20} {:<20} {:<120}".format(
-                            str(list_mode[i]),
-                            str(list_FPS[i]),
-                            str(list_features[i]),
-                        )
-                    )
-        else:
-            print(f"Superbench result of sweeping [{sweep_objective}] printed below with sorted FPS: ")
-            print("{:<20} {:<20} {:<20} {:<120}".format(
-                'Numactl Mode', 'Performance (FPS)', 'Accuracy', 'Features Applied'))
-
-            sort_index = sorted(
-                range(len(list_FPS)),
-                key=lambda k: list_FPS[k],
-                reverse=True,
-            )
-
-            for i in sort_index:
-                if list_FPS[i] != 0:
-                    print(
-                        "{:<20} {:<20} {:<20} {:<120}".format(
-                            str(list_mode[i]),
-                            str(list_FPS[i]),
-                            str(list_accuracy[i]),
-                            str(list_features[i]),
-                        )
+        for i in sort_index:
+            if list_FPS[i] != 0:
+                print(
+                    "{:<20} {:<20} {:<20} {:<120}".format(
+                        str(list_mode[i]),
+                        str(list_FPS[i]),
+                        str(list_accuracy[i]),
+                        str(list_features[i]),
                     )
+                )
 
         # for superbench report generation
         list_optimization_set_top3 = []
@@ -1003,6 +1055,7 @@ def superbench(
             code=code,
             features=features_to_generate,
             save_patch_path="intel_optimization",
+            use_inc=use_inc,
         )
         logger.info('The optimization patch was saved to "intel_optimziation.diff"')
 
@@ -1062,7 +1115,7 @@ def superbench(
                             ncore_per_instance=ncore_per_instance,
                             ninstances=ninstances,
                             bench_batch_size=bench_batch_size,
-                            eval_accuracy=eval_accuracy,
+                            use_inc=use_inc,
                         )
 
                         if dry_run:
@@ -1143,7 +1196,6 @@ def superbench(
 #     code,
 #     save_path="superbench_report.pdf",
 #     logging_level="info",
-#     eval_accuracy=False,
 #     platform="bare_metal",
 #     bare_metal_machine_type="SPR",
 # ):
@@ -1156,7 +1208,6 @@ def superbench(
 #         sweep_objective="feature",
 #         mode="throughput",
 #         logging_level=logging_level,
-#         eval_accuracy=eval_accuracy,
 #     )
 
 #     res5, res6, res7, res8 = superbench(
@@ -1164,7 +1215,6 @@ def superbench(
 #         sweep_objective="bench_config",
 #         bench_feature=res1[0],
 #         logging_level=logging_level,
-#         eval_accuracy=eval_accuracy,
 #     )
 #     res1[0] = res1[0][0:-2]
 
@@ -1223,15 +1273,14 @@ def auto_quant(
     sweep_objective="feature",
     bench_feature=[],
     mode="throughput",
-    num_benchmark_iteration=5,
+    num_benchmark_iteration=30,
     iteration_dynamic_adjust=False,
     logging_level="info",
-    cpu_conversion=True,
     cpu_set_env=True,
     ncore_per_instance=-1,  # only for "self_defined" mode
     ninstances=-1,  # only for "self_defined" mode
     bench_batch_size=-1,  # only for "self_defined" mode
-    eval_accuracy=False,
+    use_inc=False,
 ):
     return superbench(
         code,
@@ -1243,11 +1292,10 @@ def auto_quant(
         num_benchmark_iteration=num_benchmark_iteration,
         iteration_dynamic_adjust=iteration_dynamic_adjust,
         logging_level=logging_level,
-        cpu_conversion=cpu_conversion,
         cpu_set_env=cpu_set_env,
         ncore_per_instance=ncore_per_instance,  # only for "self_defined" mode
         ninstances=ninstances,  # only for "self_defined" mode
         bench_batch_size=bench_batch_size,  # only for "self_defined" mode
-        eval_accuracy=eval_accuracy,
+        use_inc=use_inc,
         auto_quant=True,
     )
