@@ -29,6 +29,13 @@ def get_all_code_path(user_input: str) -> List:
         import_path = get_imports_path(user_code_path)
     else:
         import_path = []
+
+    # if import_path intersects user_code_path, clear import_path: this is
+    # for cases where there is import of self folder and we only care about 
+    # the main file (user_code_path) itself
+    if len(list(set(user_code_path).intersection(set(import_path)))) > 0:
+        import_path = []
+
     all_code_path = user_code_path + import_path
     logger.debug(f"Number of code files to analyze: {len(all_code_path)}")
 
@@ -90,12 +97,17 @@ def get_imports_path(user_code_path: List) -> List:
     pip_name_exceptions = [
         "argparse",
         "ast",
+        "classy_vision",
         "collections",
         "contextlib",
-        "datasets",
+        "data",
         "dataclasses",
+        "dataloader",
+        "dataset",
+        "datasets",
         "einops",
         "enum",
+        "evaluate",
         "fx",
         "glob",
         "h5py",
@@ -103,17 +115,22 @@ def get_imports_path(user_code_path: List) -> List:
         "importlib",
         "inspect",
         "intel_extension_for_pytorch",
+        "itertools",
         "jax",
         "json",
         "keras",
         "libcst",
         "linecache",
         "logging",
+        "loggings",
         "math",
+        "model",
+        "models",
         "neural_coder",
         "neural_compressor",
         "numpy",
         "os",
+        "onnxruntime",
         "pandas",
         "PIL",
         "random",
@@ -121,6 +138,7 @@ def get_imports_path(user_code_path: List) -> List:
         "shutil",
         "subprocess",
         "sys",
+        "system",
         "tempfile",
         "typing",
         "tensorflow",
@@ -131,8 +149,15 @@ def get_imports_path(user_code_path: List) -> List:
         "torchdynamo",
         "tqdm",
         "traceback",
+        "trainer_qa",
         "unittest",
+        "utils_qa",
+        "vissl",
+        "warning",
         "warnings",
+        "packaging",
+        "pytorch_lightning",
+        "__future__",
     ]
 
     if globals.cache_load_transformers:
