@@ -30,7 +30,7 @@ from copy import deepcopy
 import math
 
 mx = LazyImport("mxnet")
-logger = logging.getLogger()
+logger = logging.getLogger("neural_compressor")
 
 
 @adaptor_registry
@@ -56,6 +56,7 @@ class MxNetAdaptor(Adaptor):
 
         self.ctx = mx.cpu() if framework_specific_info['device'] == 'cpu' else None
         self.benchmark = (GLOBAL_STATE.STATE == MODE.BENCHMARK)
+        self.optype_statistics = None
         assert self.ctx is not None, 'Unsupported device'
 
     @dump_elapsed_time("Pass quantize model")
@@ -320,7 +321,8 @@ class MxNetAdaptor(Adaptor):
         return collector.tensors_dicts
 
     def inspect_tensor(self, nc_model, data_x, op_list=[], iteration_list=[],
-                       inspect_type='activation', save_to_disk=False):
+                       inspect_type='activation', save_to_disk=False,
+                       save_path = None, quantization_cfg = None):
         """The function is used by tune strategy class for dumping tensor info.
 
         Args:
@@ -352,7 +354,7 @@ class MxNetAdaptor(Adaptor):
                     tensor_dict[key][tensor_name] = tensor_dict[key][tensor_name].asnumpy()
 
                 # transform to format expected by neural_compressor (assume only 1 tensor for now)
-                node, op = key
+                node = key
                 assert len(tensors) == 1, 'Multiple tensors from a single node are not supported'
                 tensor = list(tensor_dict[key].values())[0]
                 tensor_dict[key] = {node: tensor}
@@ -429,6 +431,12 @@ class MXNetQuery(QueryBackendCapability):
                 self.cur_config = None
                 raise ValueError("Please check if the format of {} follows Neural Compressor yaml schema.".
                                  format(self.cfg))
+        self._update_cfg_with_usr_definition()
+
+    def _update_cfg_with_usr_definition(self):
+        from neural_compressor.conf.pythonic_config import mxnet_config
+        if mxnet_config.precisions is not None:
+            self.cur_config['precisions']['names'] = ','.join(mxnet_config.precisions)
 
     def _get_specified_version_cfg(self, data):
         """Get the configuration for the current runtime.
