@@ -14,27 +14,52 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
+"""Direct8Bit Operator."""
 
-from .qdq_base_operator import QDQOperatorBase
-from .base_operator import QuantOperatorBase
-from .base_operator_cast import CastOperatorBase
-from neural_compressor.adaptor.ox_utils.util import QuantizedValue
+from neural_compressor.adaptor.ox_utils.operators.ops import op_registry, Operator, qop_registry, QOperator
 
-# For operators that support 8bits operations directly, and output could
-# reuse input[0]'s type, zeropoint, scale; For example,Transpose, Reshape, etc.
+@op_registry(op_types="Reshape, Transpose, Squeeze, Unsqueeze")
+class Direct8BitOperator(Operator):
+    """Direct8Bit Operator."""
 
-class Direct8BitOp(QuantOperatorBase):
     def __init__(self, onnx_quantizer, onnx_node):
-        super().__init__(onnx_quantizer, onnx_node)
+        """Initialization."""
+        super(Direct8BitOperator, self).__init__(onnx_quantizer, onnx_node)
 
-    def convert(self):
+    def quantize_check(self):
+        """Check if quantizaion can be done."""
         node = self.node
+        if not self.quantizer.is_valid_quantize_weight(node.input[0]):
+            return False
+        return True
+        
+    def quantize(self):
+        """Do quantizaion."""
+        node = self.node
+        self.quantizer.quantize_inputs(self.node, [0], direct_int8=True)
+        if not self.disable_qdq_for_node_output or self.quantizer.mode != 'qdq':
+            self.quantizer.quantize_outputs(self.node, direct_int8=True)
+        node.name = node.name + "_quant"
+
+    def convert_check(self, convert_format):
+        """Check if conversion can be done."""
+        node = self.node
+        assert convert_format in ['static'], \
+            "convert format for {} should be in ['static']".format(node.op_type)
+            
         parents = self.quantizer.model.get_parents(node)
         children = self.quantizer.model.get_children(node)
-        if len(children) == 0 and len(parents) == 0:
-            return
+        if (len(children) == 0 and len(parents) == 0) or \
+            not node.name.endswith('_quant'):
+            return False
+        return True
 
+    def convert(self, convert_format):
+        """Convert to QOperator format."""
+        node = self.node
+       
+        parents = self.quantizer.model.get_parents(node)
+        children = self.quantizer.model.get_children(node)
         if any([i.op_type == 'DequantizeLinear' for i in parents]) and \
             any([i.op_type == 'QuantizeLinear' for i in children]):
             for parent in parents:
@@ -48,27 +73,33 @@ class Direct8BitOp(QuantOperatorBase):
                     self.quantizer.model.replace_input_of_all_nodes(
                         child.output[0], node.output[0] + '_quantized')
             node.output[0] = node.output[0] + '_quantized' 
-
-class QDQDirect8BitOp(QDQOperatorBase):
-    def __init__(self, onnx_quantizer, onnx_node):
-        super().__init__(onnx_quantizer, onnx_node)
-
-    def quantize(self):
-        node = self.node
-        if not self.quantizer.is_valid_quantize_weight(node.input[0]):
-            return
-        self.quantizer.quantize_inputs(self.node, direct_int8=True)
-        if not self.disable_qdq_for_node_output or self.quantizer.mode != 'qdq':
-            self.quantizer.quantize_outputs(self.node, direct_int8=True)
-        node.name = node.name + "_quant"
-
-class DirectCast(CastOperatorBase):
-    def __init__(self, onnx_quantizer, onnx_node):
-        super().__init__(onnx_quantizer, onnx_node)
-
-    def cast(self):
+    
+    def cast(self): # pragma: no cover
+        """Cast node."""
         node = self.node
         if node.input[0] not in [i.tensor_name for i in self.quantizer.new_value_info.values()]:
             return
-        super().cast()
- 
+        self.quantizer.dtype_cast(self.node, self.dtype)
+
+@op_registry(op_types="Shape, Loop, Slice")
+class DirectCastOperator(Operator): # pragma: no cover
+    """Direct8bit Operator Cast."""
+
+    def __init__(self, onnx_quantizer, onnx_node):
+        """Initialization."""
+        super(DirectCastOperator, self).__init__(onnx_quantizer, onnx_node)
+
+    def cast(self):
+        """Cast node."""
+        node = self.node
+        if node.input[0] not in [i.tensor_name for i in self.quantizer.new_value_info.values()]:
+            return
+        self.quantizer.dtype_cast(self.node, self.dtype)
+
+@qop_registry(op_types="Reshape, Transpose, Squeeze, Unsqueeze")
+class QDirectOperator(QOperator):
+    """QDirect Operator."""
+
+    def __init__(self, onnx_node, children, initializers):
+        """Initialization."""
+        super().__init__(onnx_node, children, initializers)
\ No newline at end of file
