@@ -15,6 +15,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Insert print node Graph Rewriter."""
 
 import tensorflow as tf
 from tensorflow.core.framework import attr_value_pb2
@@ -23,19 +24,21 @@ from tensorflow.python.framework import tensor_util as tu
 from ..graph_base import GraphRewriterBase
 from neural_compressor.adaptor.tf_utils.graph_util import GraphAnalyzer
 from neural_compressor.adaptor.tf_utils.graph_util import GraphRewriterHelper as Helper
-
+from neural_compressor.adaptor.tf_utils.util import version1_gt_version2
 
 class InsertPrintMinMaxNode(GraphRewriterBase):
-    """InsertPrintMinMaxNode Pass for tensorflow sampling.
-    """
+    """InsertPrintMinMaxNode Pass for tensorflow sampling."""
 
-    def __init__(self, model, pre_node_name, post_node_name):
+    def __init__(self, model, pre_node_name, post_node_name, new_api):
+        """Intilization."""
         super().__init__(model)
         self.pre_node_name = pre_node_name
         self.post_node_name = post_node_name
         self.signature = pre_node_name + post_node_name
+        self.new_api = new_api
 
     def do_transformation(self):
+        """Insert print node in the graph to do the calibration."""
         cur_graph = GraphAnalyzer()
         cur_graph.graph = self.model
 
@@ -45,21 +48,32 @@ class InsertPrintMinMaxNode(GraphRewriterBase):
         if top_node.op == 'ConcatV2':
             for i in range(top_node.attr['N'].i):
                 insert_node_pairs.append([top_node.input[i], self.post_node_name])
-        elif top_node.op == 'BatchMatMulV2':
+        elif top_node.op in ('BatchMatMul', 'BatchMatMulV2'):
             insert_node_pairs.append([top_node.input[0], self.post_node_name])
             if graph_info[top_node.input[1]].node.op != 'Const':
-                insert_node_pairs.append([top_node.input[1], self.post_node_name])  
+                insert_node_pairs.append([top_node.input[1], self.post_node_name])
+        elif top_node.op in ('Conv2DBackpropInput', 'Conv3DBackpropInputV2'):
+            insert_node_pairs.append([top_node.input[2], self.post_node_name])
         else:
             refresh_pre_node_name = graph_info[self.pre_node_name].node.input[0]
             # Check the Conv2D could be fused with previous Pad or not.
             # If so, we need to update the pre-node name correspondingly.
             refresh_pre_node = graph_info[Helper.node_name_from_input(refresh_pre_node_name)].node
-            if refresh_pre_node.op == 'Pad' and( top_node.op == 'Conv2D' or top_node.op == 'Conv3D'):
+            if refresh_pre_node.op == 'Pad' and top_node.op in ('Conv2D', 'Conv3D'):
                 pad_const_node_name = refresh_pre_node.input[1]
                 pad_const_node = graph_info[pad_const_node_name].node
-                padding_tensor = tu.MakeNdarray(pad_const_node.attr["value"].tensor).flatten()
+                padding_tensor = None
+                if graph_info[pad_const_node_name].node.op != 'Const':
+                    if pad_const_node.op == 'DataFormatVecPermute':
+                        parent_input_node = graph_info[pad_const_node.input[0]].node
+                        if parent_input_node.op == 'Const':
+                            padding_tensor = tu.MakeNdarray( \
+                                parent_input_node.attr["value"].tensor).flatten()
+                else:
+                    padding_tensor = tu.MakeNdarray(pad_const_node.attr["value"].tensor).flatten()
                 if not any(padding_tensor) or \
-                    (any(padding_tensor) and tf.version.VERSION in ( '1.15.0-up3', '2.8.0202151')):
+                    (any(padding_tensor) and (tf.version.VERSION == '1.15.0-up3' or self.new_api)):
+                    insert_node_pairs.append([refresh_pre_node_name, self.post_node_name])
                     refresh_pre_node_name = refresh_pre_node.input[0]
 
             insert_node_pairs.append([refresh_pre_node_name, self.post_node_name])
@@ -114,17 +128,21 @@ class InsertPrintMinMaxNode(GraphRewriterBase):
                         self.pre_node_name, each_node_name)
                     min_msg = ';{}_eightbit_min_{}__print__;__min:'.format(
                         self.pre_node_name, each_node_name)
-                    src_dt = graph_info[self.pre_node_name].node.attr["T"]
+                    # workround for swish_f32, attribute T is not in the op definition
+                    if 'swish_f32' in graph_info[self.pre_node_name].node.name:
+                        src_dt=attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum)
+                    else:
+                        src_dt = graph_info[self.pre_node_name].node.attr["T"]
                 else:
                     max_msg = ';{}_eightbit_requant_range__print__;__requant_max:'.format(
                         self.pre_node_name)
                     min_msg = ';{}_eightbit_requant_range__print__;__requant_min:'.format(
                         self.pre_node_name)
-                    src_dt = graph_info[each_node_name].node.attr["T"]
-
-                if src_dt.type==0:
-                    #workround for swish_f32, attribute T is not in the op definition
-                    src_dt=attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum)
+                    # workround for swish_f32, attribute T is not in the op definition
+                    if 'swish_f32' in graph_info[each_node_name].node.op:
+                        src_dt=attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum)
+                    else:
+                        src_dt = graph_info[each_node_name].node.attr["T"]
 
                 reshape_input_node.attr["T"].CopyFrom(src_dt)
                 min_input_node.attr["T"].CopyFrom(src_dt)
@@ -149,6 +167,8 @@ class InsertPrintMinMaxNode(GraphRewriterBase):
                 if post_node_names:
                     for post_node_name in post_node_names:
                         post_node = graph_info[post_node_name].node
+                        if each_node_name not in post_node.input:
+                            continue
                         if post_node.op == 'FusedBatchNormV3':
                             if "_print_identity" in \
                                graph_info[Helper.node_name_from_input(post_node.name)].node.input[0]:
