@@ -14,7 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+"""Freeze value with calibration Graph Rewriter."""
 
 from tensorflow.core.framework import node_def_pb2
 from tensorflow.core.framework import attr_value_pb2
@@ -29,8 +29,10 @@ import numpy as np
 import re
 
 class FreezeValueTransformer(GraphRewriterBase):
-    def __init__(self, model, max_min_data, postfix, tensor_data=None, th=1, device='gpu'):
+    """Freeze Value with calibration."""
+    def __init__(self, model, max_min_data, postfix, tensor_data=None, th=1, device='gpu', itex_mode=False):
         """Free Max/Min value into QuantizeV2 op.
+
         Args:
             model (graphdef): input model
             max_min_data (string list): the string context contains max/min values.
@@ -39,6 +41,7 @@ class FreezeValueTransformer(GraphRewriterBase):
                                 which calculated by KL.
             th (float, optional): The percentage of overall data.Defaults to 0.95.
             device (string, optional): The hardware device type, 'cpu' or 'gpu'.
+            itex_mode: itex mode.
         """
         super().__init__(model)
         self.data = max_min_data
@@ -50,6 +53,7 @@ class FreezeValueTransformer(GraphRewriterBase):
             self.threshold = 0.95
         self.postfix = postfix
         self.device = device
+        self.itex_mode = itex_mode
         self.tensor_data = tensor_data
         self.cur_graph = GraphAnalyzer()
         self.cur_graph.graph = self.model
@@ -62,6 +66,10 @@ class FreezeValueTransformer(GraphRewriterBase):
         self.scale_info = {}
 
     def _get_valid_log(self):
+        """Get valid sampling log.
+
+        :return: get the valid sampling log output.
+        """
         output = []
 
         target_lines = [i.strip() for i in self.data if i.strip().find(';') != -1]
@@ -80,8 +88,8 @@ class FreezeValueTransformer(GraphRewriterBase):
         return output
 
     def _parse_max_min_log(self):
-        """
-        Parse the max_ming log file
+        """Parse the max_ming log file.
+
         :return: get the node name and value mapping
         """
         print_suffix = "__print__"
@@ -113,8 +121,8 @@ class FreezeValueTransformer(GraphRewriterBase):
         return res
 
     def _parse_requantization_ranges(self):
-        """
-        Parse the max_min log to get requantization values
+        """Parse the max_min log to get requantization values.
+
         :return: dict saved the result
         """
         res = {}
@@ -166,8 +174,8 @@ class FreezeValueTransformer(GraphRewriterBase):
         return res
 
     def generate_output_graph(self, max_name_value):
-        """
-        Generate transformed graph for freeze_max/freeze_min transformation.
+        """Generate transformed graph for freeze_max/freeze_min transformation.
+
         :param max_name_value: target values
         :return: transformed graph
         """
@@ -188,7 +196,7 @@ class FreezeValueTransformer(GraphRewriterBase):
                     dtypes.float32, [])))
             output_node_name = self.graph_info[node_name].outputs[0]
 
-            if node_name in self.cur_graph.parent_frame_details and \
+            if not self.itex_mode and node_name in self.cur_graph.parent_frame_details and \
                self.cur_graph.parent_frame_details[node_name]:   # pragma: no cover      
                 new_node_enter_node = Helper.create_node(
                     'Enter', new_node.name+'_enter', [new_node.name])
@@ -220,25 +228,34 @@ class FreezeValueTransformer(GraphRewriterBase):
         return GraphAnalyzer().dump_graph(), self.scale_info
 
     def generate_output_graph_ranges(self, max_name_value):
-        """
-        Generate transformed graph for freeze_max/freeze_min transformation.
+        """Generate transformed graph for freeze_max/freeze_min transformation.
+
         :param max_name_value: target values
         :return: transformed graph
         """
         for node_name, value in max_name_value.items():
             bn_node_name = node_name.replace('eightbit_requant_range', 'eightbit_quantized_bn')
+            in_node_name = node_name.replace('eightbit_requant_range', 'eightbit_quantized_in')
             if not self.graph_info.get(bn_node_name) or \
                 not bn_node_name.endswith('_eightbit_quantized_bn'):
                 bn_node_name = None
+            if not self.graph_info.get(in_node_name) or \
+                not in_node_name.endswith('_eightbit_quantized_in'):
+                in_node_name = None
             if node_name not in self.graph_info \
-                and bn_node_name not in self.graph_info:
+                and bn_node_name not in self.graph_info \
+                    and in_node_name not in self.graph_info:
                 continue
 
             min_node = node_def_pb2.NodeDef()
             min_node.op = "Const"
             min_node_postfix = "/frozen_min"
-            min_node.name = bn_node_name + "/frozen_bn_output_min" if bn_node_name \
-                else node_name + min_node_postfix
+            if bn_node_name:
+                min_node.name = bn_node_name + "/frozen_bn_output_min"
+            elif in_node_name:
+                min_node.name = in_node_name + "/frozen_in_output_min"
+            else:
+                min_node.name = node_name + min_node_postfix
             min_node.attr["dtype"].CopyFrom(
                 attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
             min_node.attr["value"].CopyFrom(
@@ -249,8 +266,12 @@ class FreezeValueTransformer(GraphRewriterBase):
             max_node = node_def_pb2.NodeDef()
             max_node.op = "Const"
             max_node_postfix = "/frozen_max"
-            max_node.name = bn_node_name + "/frozen_bn_output_max" if bn_node_name \
-                else node_name + max_node_postfix
+            if bn_node_name:
+                max_node.name = bn_node_name + "/frozen_bn_output_max"
+            elif in_node_name:
+                max_node.name = in_node_name + "/frozen_in_output_max"
+            else:
+                max_node.name = node_name + max_node_postfix
             max_node.attr["dtype"].CopyFrom(
                 attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
             max_node.attr["value"].CopyFrom(
@@ -269,7 +290,18 @@ class FreezeValueTransformer(GraphRewriterBase):
                     [Helper.node_name_from_input(bn_node_name)],
                     bn_node_name + '_input8_output_max'
                 )
-            elif node_name in self.cur_graph.parent_frame_details and \
+            elif in_node_name:
+                self.cur_graph.replace_const_node(
+                    min_node,
+                    [Helper.node_name_from_input(in_node_name)],
+                    in_node_name + '_input7_output_min'
+                )
+                self.cur_graph.replace_const_node(
+                    max_node,
+                    [Helper.node_name_from_input(in_node_name)],
+                    in_node_name + '_input8_output_max'
+                )
+            elif not self.itex_mode and node_name in self.cur_graph.parent_frame_details and \
                  self.cur_graph.parent_frame_details[node_name]:         # pragma: no cover
                 output_node_name = self.graph_info[node_name].outputs[0]
                 min_node_enter_node = Helper.create_node(
@@ -336,6 +368,7 @@ class FreezeValueTransformer(GraphRewriterBase):
         return GraphAnalyzer().dump_graph(), self.scale_info
 
     def do_transformation(self):
+        """Apply the transformation of freeze value."""
         if self.postfix == '__requant_min_max':
             range_data = self._parse_requantization_ranges()
 
