@@ -14,6 +14,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Fuse QuantizedConv Requantize/Dequantize Graph Rewriter."""
+
 import tensorflow as tf
 from tensorflow.python.framework import tensor_util
 from tensorflow.core.framework import attr_value_pb2
@@ -25,16 +27,17 @@ from neural_compressor.adaptor.tf_utils.graph_util import GraphAnalyzer
 from neural_compressor.adaptor.tf_utils.graph_util import GraphRewriterHelper as Helper
 
 class FuseConvRequantizeTransformer(GraphRewriterBase):
-    """Fuse Quantized Conv Op with the successor Requantize Op.
-    """
+    """Fuse Quantized Conv Op with the successor Requantize Op."""
     fuse_patterns = [[
         "QuantizedConv2DWithBiasAndRelu",
         "QuantizedDepthwiseConv2DWithBiasAndRelu",
         "QuantizedConv2DWithBias",
         "QuantizedDepthwiseConv2DWithBias",
-        "_QuantizedConv2D",
-        "_QuantizedDepthwiseConv2D",
-        "_QuantizedConv3D",
+        "_FusedQuantizedConv2D",
+        "_FusedQuantizedDepthwiseConv2D",
+        "_FusedQuantizedConv3D",
+        "_FusedQuantizedDeconv2D",
+        "_FusedQuantizedDeconv3D"
     ], ['RequantizePerChannel', 'Requantize'], ('Dequantize',)]
    
     fuse_sum_op_types = (
@@ -46,10 +49,11 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
         )
 
     sum_pattern = [["QuantizedConv2DWithBiasSumAndRelu", "QuantizedConv2DWithBiasReluAndSum", \
-                    "_QuantizedDepthwiseConv2D", "_QuantizedConv2D", "_QuantizedConv3D"],
+                    "_FusedQuantizedDepthwiseConv2D", "_FusedQuantizedConv2D", "_FusedQuantizedConv3D"],
                      ['RequantizePerChannel', 'Requantize']]
 
     def __init__(self, model, device='cpu', new_api=False):
+        """Initilization."""
         super().__init__(model)
         self.device = device
         self.graph_analyzer = GraphAnalyzer()
@@ -61,8 +65,10 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
 
     def do_transformation(self):
         """Fuse the quantized op with the following requantize op.
-            The transformation has two stages, the first step is to fuse the patterns
-            defined in self.fuse_patterns and the last step is to fuse the self.sum_patterns.
+
+        The transformation has two stages, the first step is to fuse the patterns
+        defined in self.fuse_patterns and the last step is to fuse the self.sum_patterns.
+
         Returns:
             [graphdef]: the optimized graphdef object
         """
@@ -83,7 +89,8 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             quantized_node = self.graph_info[quantized_node_name].node
             if not self.new_api and quantized_node.op == "QuantizedDepthwiseConv2DWithBias":
                 continue
-            if i[-1][0] in ('_QuantizedDepthwiseConv2D', '_QuantizedConv2D', '_QuantizedConv3D'):
+            if i[-1][0] in ('_FusedQuantizedDepthwiseConv2D', '_FusedQuantizedConv2D', '_FusedQuantizedConv3D', \
+                "_FusedQuantizedDeconv2D", "_FusedQuantizedDeconv3D"):
                 if str(quantized_node.attr['fused_ops'].list.s).find('Sum') != -1:
                     continue
                 #else:
@@ -91,7 +98,8 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             requantize_node_name = i[1]
             requantize_node = self.graph_info[requantize_node_name].node
             if i[-1][-1] == 'Dequantize' and self.new_api and \
-                i[0] in ('_QuantizedDepthwiseConv2D', '_QuantizedConv2D', '_QuantizedConv3D'):
+                i[0] in ('_FusedQuantizedDepthwiseConv2D', '_FusedQuantizedConv2D', '_FusedQuantizedConv3D', \
+                    "_FusedQuantizedDeconv2D", "_FusedQuantizedDeconv3D"):
                 dequantize_node_name = i[2]
             else:
                 dequantize_node_name = None
@@ -103,39 +111,43 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             new_node = node_def_pb2.NodeDef()
             if self.new_api:
                 if i[-1][0] == 'QuantizedConv2DWithBiasAndRelu':
-                    new_node.op = '_QuantizedConv2D'
+                    new_node.op = '_FusedQuantizedConv2D'
                     self.fused_ops= [b"BiasAdd", b"Relu", b"Requantize"]
                     self.output_types= [
                                         requantize_node.attr['out_type'].type,
                                         dtypes.float32.as_datatype_enum,
                                         dtypes.float32.as_datatype_enum]
                 elif i[-1][0] == 'QuantizedConv2DWithBias':
-                    new_node.op = '_QuantizedConv2D'
+                    new_node.op = '_FusedQuantizedConv2D'
                     self.fused_ops = [b"BiasAdd", b"Requantize"]
                     self.output_types = [
                                         requantize_node.attr['out_type'].type,
                                         dtypes.float32.as_datatype_enum,
                                         dtypes.float32.as_datatype_enum]
                 elif i[-1][0] == 'QuantizedDepthwiseConv2DWithBias':
-                    new_node.op = '_QuantizedDepthwiseConv2D'
+                    new_node.op = '_FusedQuantizedDepthwiseConv2D'
                     self.fused_ops = [b"BiasAdd", b"Requantize"]
                     self.output_types = [
                                         requantize_node.attr['out_type'].type,
                                         dtypes.float32.as_datatype_enum,
                                         dtypes.float32.as_datatype_enum]
                 elif i[-1][0] == 'QuantizedDepthwiseConv2DWithBiasAndRelu':
-                    new_node.op = '_QuantizedDepthwiseConv2D'
+                    new_node.op = '_FusedQuantizedDepthwiseConv2D'
                     self.fused_ops= [b"BiasAdd", b"Relu", b"Requantize"]
                     self.output_types= [
                                         requantize_node.attr['out_type'].type,
                                         dtypes.float32.as_datatype_enum,
                                         dtypes.float32.as_datatype_enum]
-                elif quantized_node_op == '_QuantizedConv2D':
-                    new_node.op = '_QuantizedConv2D'
-                elif quantized_node_op == '_QuantizedDepthwiseConv2D':
-                    new_node.op = '_QuantizedDepthwiseConv2D'
-                elif quantized_node_op == '_QuantizedConv3D':
-                    new_node.op = '_QuantizedConv3D'
+                elif quantized_node_op == '_FusedQuantizedConv2D':
+                    new_node.op = '_FusedQuantizedConv2D'
+                elif quantized_node_op == '_FusedQuantizedDepthwiseConv2D':
+                    new_node.op = '_FusedQuantizedDepthwiseConv2D'
+                elif quantized_node_op == '_FusedQuantizedConv3D':
+                    new_node.op = '_FusedQuantizedConv3D'
+                elif quantized_node_op == '_FusedQuantizedDeconv2D':
+                    new_node.op = "_FusedQuantizedDeconv2D"
+                elif quantized_node_op == '_FusedQuantizedDeconv3D':
+                    new_node.op = "_FusedQuantizedDeconv3D"
             else:
                 new_node.op = quantized_node_op + "AndRequantize"
             new_node.name = requantize_node_name
@@ -154,17 +166,24 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                 new_node.attr["alpha"].CopyFrom(quantized_node.attr['alpha'])
             if 'Tsummand' in quantized_node.attr:
                 new_node.attr["Tsummand"].CopyFrom(quantized_node.attr['Tsummand'])
+            if 'data_format' in quantized_node.attr:
+                new_node.attr["data_format"].CopyFrom(quantized_node.attr['data_format'])
 
             parent_node_name = Helper.node_name_from_input(quantized_node.input[0])
-            max_filter_node = self.graph_info[new_node.input[-1]].node
-            min_filter_node = self.graph_info[new_node.input[-2]].node
+            if new_node.op in ('_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
+                max_filter_node = self.graph_info[new_node.input[-3]].node
+                min_filter_node = self.graph_info[new_node.input[-4]].node
+            else:
+                max_filter_node = self.graph_info[new_node.input[-1]].node
+                min_filter_node = self.graph_info[new_node.input[-2]].node
             last_node = self.graph_info[new_node.input[0]].node
             new_node.input.append(requested_output_min_name)
-            new_node.input.append(requested_output_max_name)           
- 
-            if last_node.op.find('Requantize') != -1 or ((last_node.op.find('QuantizeV2') != -1 or \
-                                                            last_node.op.find('QuantizedConv2D') != -1) \
-                                                                and len(quantized_node.attr['fused_ops'].list.s) > 0):          
+            new_node.input.append(requested_output_max_name)
+
+            if (last_node.op.find('Requantize') != -1 or \
+                ((last_node.op.find('QuantizeV2') != -1 or \
+                  last_node.op.find('QuantizedConv2D') != -1))) and \
+                  len(quantized_node.attr['fused_ops'].list.s) > 0:
                 bias_node = self.graph_info[new_node.input[2]].node
                 max_input_node = self.graph_info[last_node.input[-1]].node
                 min_input_node = self.graph_info[last_node.input[-2]].node
@@ -213,26 +232,43 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                 elif quantized_node.attr["padding"].s == b"EXPLICIT":
                     new_node.attr["explicit_paddings"].CopyFrom(quantized_node.attr['padding_list'])
             elif "explicit_paddings" in quantized_node.attr:
-                new_node.attr["explicit_paddings"].CopyFrom(quantized_node.attr['explicit_paddings'])           
+                new_node.attr["explicit_paddings"].CopyFrom(quantized_node.attr['explicit_paddings'])
             if "dilations" in quantized_node.attr:
                 new_node.attr["dilations"].CopyFrom(quantized_node.attr['dilations'])
             
-            if self.new_api and new_node.op in ('_QuantizedConv2D', '_QuantizedDepthwiseConv2D'):
+            if self.new_api and new_node.op in ('_FusedQuantizedConv2D', '_FusedQuantizedDepthwiseConv2D', \
+                                                '_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
                 input_data_type = dtypes.qint8 if new_node.attr["Tinput"].type == dtypes.qint8 else dtypes.quint8
-                Helper.set_attr_type_list(new_node, 'Thost_inputs', [
-                    input_data_type.as_datatype_enum,
-                    dtypes.qint8.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 \
-                                                                            else dtypes.qint32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                    dtypes.float32.as_datatype_enum,
-                ])
+                if new_node.op in ('_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        dtypes.int32.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        input_data_type.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 \
+                                                        else dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum
+                    ])
+                else:
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        input_data_type.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 \
+                                                        else dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum
+                    ])
 
-                if quantized_node_op not in ('_QuantizedConv2D', '_QuantizedDepthwiseConv2D'): 
+                if quantized_node_op not in ('_FusedQuantizedConv2D', '_FusedQuantizedDepthwiseConv2D', \
+                                             '_FusedQuantizedDeconv2D','_FusedQuantizedDeconv3D'):
                     Helper.set_attr_type_list(new_node, 'Thost_outputs', self.output_types)
                     new_node.attr["Tsummand"].CopyFrom(attr_value_pb2.AttrValue(type=self.output_types[0]))
                 else:
@@ -304,21 +340,35 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                                               dtypes.float32.as_datatype_enum ])
                         Helper.set_attr_dtype(new_node, "out_type", \
                             dtype_map_dict[requantize_node.attr['out_type'].type])
-                        Helper.set_attr_dtype(new_node, "Tsummand", \
-                            dtype_map_dict[requantize_node.attr['out_type'].type])
+                        if new_node.op not in ('_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
+                            Helper.set_attr_dtype(new_node, "Tsummand", \
+                                dtype_map_dict[requantize_node.attr['out_type'].type])
                     elif len(quantized_node.attr['fused_ops'].list.s) == 0:
-                        Helper.set_attr_type_list(new_node, 'Thost_inputs', [
-                            input_data_type.as_datatype_enum,
-                            dtypes.qint8.as_datatype_enum,
-                            #dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 \
-                                                                            # else dtypes.qint32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                            dtypes.float32.as_datatype_enum,
-                        ])
+                        if new_node.op in ('_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
+                            Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                                dtypes.int32.as_datatype_enum,
+                                dtypes.qint8.as_datatype_enum,
+                                input_data_type.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum
+                            ])
+                        else:
+                            Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                                input_data_type.as_datatype_enum,
+                                dtypes.qint8.as_datatype_enum,
+                                #dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 \
+                                                                                # else dtypes.qint32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum,
+                                dtypes.float32.as_datatype_enum
+                            ])
                         self.fused_ops= [b"Requantize"]
                         Helper.set_attr_type_list(new_node, 'Thost_outputs', [
                                               requantize_node.attr['out_type'].type,
@@ -326,14 +376,15 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                                               dtypes.float32.as_datatype_enum ])
                         Helper.set_attr_dtype(new_node, "out_type", \
                             dtype_map_dict[requantize_node.attr['out_type'].type])
-                        Helper.set_attr_dtype(new_node, "Tsummand", \
-                            dtype_map_dict[requantize_node.attr['out_type'].type])
+                        if new_node.op not in ('_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
+                            Helper.set_attr_dtype(new_node, "Tsummand", \
+                                dtype_map_dict[requantize_node.attr['out_type'].type])
                 Helper.set_attr_string_list(new_node, 'fused_ops', self.fused_ops)
-     
+
             if "_kernel" in quantized_node.attr:
                 new_node.attr["_kernel"].CopyFrom(quantized_node.attr['_kernel'])
 
-            if new_node.op in ('_QuantizedConv3D'):
+            if new_node.op in ('_FusedQuantizedConv3D'):
                 input_data_type = dtypes.qint8 if new_node.attr["Tinput"].type == dtypes.qint8 else dtypes.quint8
                 if len(quantized_node.attr['fused_ops'].list.s) == 0:
                     Helper.set_attr_string_list(new_node, 'fused_ops', [ b'Requantize'])
@@ -375,6 +426,34 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                         dtypes.float32.as_datatype_enum,
                         dtypes.float32.as_datatype_enum,
                     ])
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd", b"LeakyRelu"]):
+                    Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'LeakyRelu', b'Requantize'])
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        input_data_type.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 else \
+                                                                                    dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                    ])
+                elif str(quantized_node.attr['fused_ops'].list.s) == str([b"BiasAdd", b"Elu"]):
+                    Helper.set_attr_string_list(new_node, 'fused_ops', [b'BiasAdd', b'Elu', b'Requantize'])
+                    Helper.set_attr_type_list(new_node, 'Thost_inputs', [
+                        input_data_type.as_datatype_enum,
+                        dtypes.qint8.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum if new_node.attr["Tbias"].type == dtypes.float32 else \
+                                                                                    dtypes.qint32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                        dtypes.float32.as_datatype_enum,
+                    ])
 
                 Helper.set_attr_type_list(new_node, 'Thost_outputs', [
                                       requantize_node.attr['out_type'].type,
@@ -384,7 +463,7 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                     dtype_map_dict[requantize_node.attr['out_type'].type])
                 Helper.set_attr_dtype(new_node, "Tsummand", \
                     dtype_map_dict[requantize_node.attr['out_type'].type])
-                 
+
             if quantized_node.op == "QuantizedConv2D" or \
                     quantized_node.op == "QuantizedConv2DWithBias" or \
                     quantized_node.op == "QuantizedDepthwiseConv2D" or \
@@ -394,9 +473,11 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             elif quantized_node.op == "QuantizedConv2DWithBiasAndRelu" or \
                  quantized_node.op == "QuantizedDepthwiseConv2DWithBiasAndRelu":
                 new_node.attr["out_type"].CopyFrom(attr_value_pb2.AttrValue(type=uint8_type))
-            elif new_node.op not in ('_QuantizedConv2D', '_QuantizedDepthwiseConv2D', '_QuantizedConv3D'):
+            elif new_node.op not in ('_FusedQuantizedConv2D', '_FusedQuantizedDepthwiseConv2D', \
+                        '_FusedQuantizedConv3D', '_FusedQuantizedDeconv2D', '_FusedQuantizedDeconv3D'):
                 new_node.attr["out_type"].CopyFrom(attr_value_pb2.AttrValue(type=uint8_type))
-
+            elif new_node.op in ("_FusedQuantizedDeconv2D", "_FusedQuantizedDeconv3D"):
+                new_node.attr["out_type"].CopyFrom(attr_value_pb2.AttrValue(type=int8_type))
             old_input_name = dequantize_node_name if dequantize_node_name else requantize_node_name
             self.graph_analyzer.replace_single_node(
                 new_node, [parent_node_name], quantized_node_name,
@@ -407,7 +488,7 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
         for i in target_nodes:
             quantized_node_name = i[0]
             quantized_node = self.graph_info[quantized_node_name].node
-            if i[-1][0] in ('_QuantizedDepthwiseConv2D', '_QuantizedConv2D', '_QuantizedConv3D'):
+            if i[-1][0] in ('_FusedQuantizedDepthwiseConv2D', '_FusedQuantizedConv2D', '_FusedQuantizedConv3D'):
                 if quantized_node.attr['fused_ops'].list.s not in self.fuse_sum_op_types:
                    continue
                 #else:
@@ -424,14 +505,14 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
 
             if self.new_api:
                 if i[-1][0] in ('QuantizedConv2DWithBiasSumAndRelu',) :
-                    new_node.op = '_QuantizedConv2D'
+                    new_node.op = '_FusedQuantizedConv2D'
                     self.fused_ops= [b"BiasAdd", b"Sum", b"Relu", b"Requantize"]
-                elif i[-1][0] == '_QuantizedConv2D':
-                    new_node.op = '_QuantizedConv2D'
-                elif quantized_node_op == '_QuantizedDepthwiseConv2D':
-                    new_node.op = '_QuantizedDepthwiseConv2D'
-                elif i[-1][0] == '_QuantizedConv3D':
-                    new_node.op = '_QuantizedConv3D'
+                elif i[-1][0] == '_FusedQuantizedConv2D':
+                    new_node.op = '_FusedQuantizedConv2D'
+                elif quantized_node_op == '_FusedQuantizedDepthwiseConv2D':
+                    new_node.op = '_FusedQuantizedDepthwiseConv2D'
+                elif i[-1][0] == '_FusedQuantizedConv3D':
+                    new_node.op = '_FusedQuantizedConv3D'
             else:
                 new_node.op = quantized_node_op + "AndRequantize"
 
@@ -482,7 +563,7 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
             new_node.attr["Tbias"].CopyFrom(attr_value_pb2.AttrValue(type=float32_type))
             new_node.attr["Tsummand"].CopyFrom(attr_value_pb2.AttrValue(type=summand_op_type))
 
-            if new_node.op in ('_QuantizedConv2D', '_QuantizedDepthwiseConv2D', '_QuantizedConv3D'):
+            if new_node.op in ('_FusedQuantizedConv2D', '_FusedQuantizedDepthwiseConv2D', '_FusedQuantizedConv3D'):
                 original_input = list(new_node.input)
                 new_input = []
                 new_input.extend(original_input[:3])
@@ -517,8 +598,6 @@ class FuseConvRequantizeTransformer(GraphRewriterBase):
                     self.fused_ops = [b'BiasAdd', b'LeakyRelu', b'Sum', b'Requantize']
                 elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'Relu', b'Sum']):
                     self.fused_ops = [b'BiasAdd', b'Relu', b'Sum', b'Requantize']
-                elif str(quantized_node.attr['fused_ops'].list.s) == str([b'BiasAdd', b'LeakyRelu', b'Sum']):
-                    self.fused_ops = [b'BiasAdd', b'LeakyRelu', b'Sum', b'Requantize']
                     #Current fusion requires summand has same dtype as output if output is qint8
                     Helper.set_attr_dtype(new_node, "Tsummand", \
                         dtype_map_dict[requantize_node.attr['out_type'].type])
