@@ -14,6 +14,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Quantize MatMul."""
+
 import numpy as np
 
 from tensorflow.core.framework import graph_pb2
@@ -25,13 +27,16 @@ from .quantize_graph_base import QuantizeNodeBase
 from tensorflow.python.framework import tensor_util
 
 class FuseNodeStartWithMatmul(QuantizeNodeBase):
+    """Quantize MatMul and apply the fusion."""
 
     def __init__(self, **kwargs):
+        """Initilization."""
         super().__init__(**kwargs)
 
         self.sorted_patterns = sorted(self.patterns,
                                       key=lambda i: len(i),
                                       reverse=True)
+        self.exclude_matmul_name = []
         self.fusion_op_type = set(fusion[0] for fusion in self.patterns)
         self.fusion_mapping = {
             'MatMulBiasAdd': self.apply_matmul_biasadd_fusion,
@@ -41,6 +46,7 @@ class FuseNodeStartWithMatmul(QuantizeNodeBase):
         }
 
     def apply_matmul_biasadd_relu_fusion(self, match_node_name):
+        """Apply the MatMul BiasAdd Relu fusion."""
         matched_node = self.node_name_mapping[match_node_name[0]]
         control_inputs, normal_inputs = self._get_node_input(matched_node.node.name)
         weight_name = normal_inputs[1]
@@ -137,6 +143,7 @@ class FuseNodeStartWithMatmul(QuantizeNodeBase):
         return match_node_name
 
     def apply_matmul_biasadd_fusion(self, match_node_name):
+        """Apply MatMul BiasAdd fusion."""
         skip_node_name = match_node_name[1:]
         matched_node = self.node_name_mapping[match_node_name[0]]
         control_inputs, normal_inputs = self._get_node_input(
@@ -150,10 +157,11 @@ class FuseNodeStartWithMatmul(QuantizeNodeBase):
             self.output_graph = self.input_graph
             return []
 
-        #TODO Remove below two lines once the TF enabled the QuantizedMatMul while
-        # transpose_a/transpose_a could be set to True.
+        #TODO Remove below two lines once the TF enabled the old QuantizedMatMul while
+        # transpose_a/transpose_b could be set to True.
         if matched_node.node.attr["transpose_a"].b == True or \
-            matched_node.node.attr["transpose_b"].b == True:
+                matched_node.node.attr["transpose_b"].b == True:
+            self.exclude_matmul_name.append(match_node_name[0])
             self.output_graph = self.input_graph
             return []
 
@@ -251,11 +259,13 @@ class FuseNodeStartWithMatmul(QuantizeNodeBase):
         return match_node_name
 
     def get_longest_fuse(self):
+        """Get the longest fusion pattern."""
         self._get_op_list()
         matched_rule, _ = self._is_match(self.sorted_patterns)
         return matched_rule
 
     def apply_the_transform(self):
+        """Quantize MatMul and apply the fusion pattern."""
         self._get_op_list()
         matched_rule, matched_node_name = self._is_match(self.sorted_patterns)
 
@@ -273,6 +283,6 @@ class FuseNodeStartWithMatmul(QuantizeNodeBase):
             self._reset_output_node_maps()
             if self.remove_redundant_quant_flag:
                 self.output_graph = self.remove_redundant_quantization(self.output_graph)
-            return self.output_graph, matched_nodes
+            return self.output_graph, matched_nodes, self.exclude_matmul_name
 
-        return self.input_graph, []
+        return self.input_graph, [], []
