@@ -1,3 +1,4 @@
+
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
@@ -15,11 +16,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""The SigOpt Tuning Strategy provides support for the quantization process."""
 import copy
 from neural_compressor.utils import logger
+from neural_compressor.utils.utility import LazyImport
 from neural_compressor.strategy.strategy import strategy_registry, TuneStrategy
-from sigopt import Connection
+from collections import OrderedDict
+from neural_compressor.strategy.utils.tuning_sampler import OpWiseTuningSampler
+from neural_compressor.strategy.utils.tuning_structs import OpTuningConfig
 
+sigopt = LazyImport('sigopt')
 
 @strategy_registry
 class SigOptTuneStrategy(TuneStrategy):
@@ -69,6 +75,7 @@ class SigOptTuneStrategy(TuneStrategy):
 
     def __init__(self, model, conf, q_dataloader, q_func=None,
                  eval_dataloader=None, eval_func=None, dicts=None, q_hooks=None):
+        """Initialize the SigOpt tuning strategy if the user specified to use it."""
         super().__init__(
             model,
             conf,
@@ -78,7 +85,20 @@ class SigOptTuneStrategy(TuneStrategy):
             eval_func,
             dicts,
             q_hooks)
-
+        strategy_name = conf.usr_cfg.tuning.strategy.name
+        if strategy_name.lower() == "sigopt":
+            try:
+                import sigopt
+            except ImportError:
+                try:
+                    import subprocess
+                    import sys
+                    subprocess.check_call([sys.executable, "-m", "pip", "install", "sigopt"])
+                    import sigopt # pylint: disable=import-error
+                except:
+                    assert False, "Unable to import sigopt from the local environment."
+        else:
+            pass
         # SigOpt init
         client_token = conf.usr_cfg.tuning.strategy.sigopt_api_token
         self.project_id = conf.usr_cfg.tuning.strategy.sigopt_project_id
@@ -105,36 +125,24 @@ class SigOptTuneStrategy(TuneStrategy):
         else:
            logger.info("Experiment name is {}.".format(self.experiment_name))
 
-        self.conn = Connection(client_token)
+        self.conn = sigopt.Connection(client_token)
         self.experiment = None
 
     def params_to_tune_configs(self, params):
-        op_cfgs = {}
-        op_cfgs['op'] = {}
-        for op, configs in self.opwise_quant_cfgs.items():
-            if len(configs) > 1:
-                value = int(params[op[0]])
-                if value == len(configs):
-                    value = len(configs) - 1
-                op_cfgs['op'][op] = copy.deepcopy(configs[value])
-            elif len(configs) == 1:
-                op_cfgs['op'][op] = copy.deepcopy(configs[0])
+        """Get the parameters of the tuning strategy."""
+        op_tuning_cfg = {}
+        calib_sampling_size_lst = self.tuning_space.root_item.get_option_by_name('calib_sampling_size').options
+        for op_name_type, configs in self.op_configs.items():
+            if len(configs) == 1:
+                op_tuning_cfg[op_name_type] = configs[0]
             else:
-                op_cfgs['op'][op] = copy.deepcopy(self.opwise_tune_cfgs[op][0])
-        if len(self.calib_iter) > 1:
-            value = int(params['calib_iteration'])
-            if value == len(self.calib_iter):
-                value = len(configs) - 1
-            op_cfgs['calib_iteration'] = int(self.calib_iter[value])
-        else:
-            op_cfgs['calib_iteration'] = int(self.calib_iter[0])
-        return op_cfgs
+                op_tuning_cfg[op_name_type] = configs[min(len(configs) - 1, int(params[op_name_type[0]]))]
+        calib_sampling_size = calib_sampling_size_lst[min(len(configs) - 1, int(params['calib_sampling_size']))]
+        op_tuning_cfg['calib_sampling_size'] = calib_sampling_size
+        return op_tuning_cfg
 
     def next_tune_cfg(self):
-        """The generator of yielding next tuning config to traverse by concrete strategies
-           according to last tuning result.
-
-        """
+        """Yielding the tuning config to traverse by concreting strategies according to last tuning result."""
         while self.experiment.progress.observation_count < self.experiment.observation_budget:
             suggestion = self.conn.experiments(self.experiment.id).suggestions().create()
             yield self.params_to_tune_configs(suggestion.assignments)
@@ -149,15 +157,16 @@ class SigOptTuneStrategy(TuneStrategy):
             self.experiment = self.conn.experiments(self.experiment.id).fetch()
 
     def get_acc_target(self, base_acc):
+        """Get the tuning target of the accuracy ceiterion."""
         if self.cfg.tuning.accuracy_criterion.relative:
             return base_acc * (1. - self.cfg.tuning.accuracy_criterion.relative)
         else:
             return base_acc - self.cfg.tuning.accuracy_criterion.absolute
 
     def traverse(self):
-        """The main traverse logic, which could be override by some concrete strategy which needs
-           more hooks.
-           This is SigOpt version of traverse -- with additional constraints setting to HPO.
+        """The main traverse logic, which could be override by some concrete strategy which needs more hooks.
+
+        This is SigOpt version of traverse -- with additional constraints setting to HPO.
         """
         #get fp32 model baseline
         if self.baseline is None:
@@ -202,14 +211,44 @@ class SigOptTuneStrategy(TuneStrategy):
                 break
 
     def create_exp(self, acc_target):
+        """Set the config for the experiment."""
         params = []
-        for op, configs in self.opwise_quant_cfgs.items():
+        from copy import deepcopy
+        tuning_space = self.tuning_space
+        initial_op_tuning_cfg = {}
+        for item in tuning_space.root_item.options:
+            if item.item_type == 'op':
+                op_name, op_type = item.name
+                initial_op_tuning_cfg[item.name] = OpTuningConfig(op_name, op_type, 'fp32', tuning_space)
+        calib_sampling_size_lst = tuning_space.root_item.get_option_by_name('calib_sampling_size').options
+        # step1. collect the ops that support static and dynamic
+        quant_mode_wise_items = OrderedDict()
+        query_order = ['static', 'dynamic', 'bf16', 'fp32']
+        pre_items = set()
+        for quant_mode in query_order:
+            items = tuning_space.query_items_by_quant_mode(quant_mode)
+            filtered_items = [item for item in items if item not in pre_items]
+            pre_items = pre_items.union(set(items))
+            quant_mode_wise_items[quant_mode] = filtered_items
+
+        def initial_op_quant_mode(items_lst, target_quant_mode, op_item_dtype_dict):
+            """Initialize the op tuning mode."""
+            for item in items_lst:
+                op_item_dtype_dict[item.name] = target_quant_mode
+
+        op_item_dtype_dict = OrderedDict()
+        for quant_mode, quant_mode_items in quant_mode_wise_items.items():
+            initial_op_quant_mode(quant_mode_items, quant_mode, op_item_dtype_dict)
+
+        op_wise_pool = OpWiseTuningSampler(tuning_space, [], [], 
+                                           op_item_dtype_dict, initial_op_tuning_cfg)
+        self.op_configs = op_wise_pool.get_opwise_candidate()
+        for op, configs in self.op_configs.items():
             if len(configs) > 1:
-                params.append(dict(name=op[0], type='int',\
-                              bounds=dict(min=0, max=len(configs) - 1)))
-        if len(self.calib_iter) > 1:
-            params.append(dict(name='calib_iteration', type='int',\
-                          bounds=dict(min=0, max=len(self.calib_iter) - 1)))
+                params.append(dict(name=op[0], type='int',
+                    bounds=dict(min=0, max=len(configs) - 1)))
+        params.append(dict(name='calib_sampling_size', type='int',
+                      bounds=dict(min=0, max=len(calib_sampling_size_lst) - 1)))
         experiment = self.conn.experiments().create(
             name=self.experiment_name,
             parameters=params,
