@@ -15,6 +15,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+
+"""Intel® Neural Compressor: An open-source Python library supporting popular model compression techniques."""
+
 from .component import Component
 from .quantization import Quantization
 from .pruning import Pruning
@@ -24,6 +27,8 @@ from .mixed_precision import MixedPrecision
 from .model_conversion import ModelConversion
 from .distillation import Distillation
 from .nas import NAS
+from . import export
 
 __all__ = ['Component', 'Quantization', 'Pruning', 'Benchmark', 'Graph_Optimization', \
-           'GraphOptimization', 'ModelConversion', 'Distillation', 'NAS', 'MixedPrecision']
+           'GraphOptimization', 'ModelConversion', 'Distillation', 'NAS', 'MixedPrecision', \
+           'export']
