@@ -14,6 +14,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+# ==============================================================================
+"""Dummy dataset for dummy_v2/sparse_dummy_v2 data generation on multiple framework backends."""
 
 import sys
 from .dataset import dataset_registry, IterableDataset
@@ -24,45 +26,41 @@ from functools import reduce
 mx = LazyImport('mxnet')
 torch = LazyImport('torch')
 
-@dataset_registry(dataset_type="dummy_v2", framework="tensorflow, inteltensorflow, tensorflow_itex, \
+@dataset_registry(dataset_type="dummy_v2", framework="tensorflow, tensorflow_itex, \
                                                       onnxrt_qlinearops, onnxrt_integerops, \
                                                       pytorch, pytorch_ipex, pytorch_fx, mxnet",
                                                       dataset_format='')
 class DummyDataset(IterableDataset):
     """Dataset used for dummy_v2 data generation.
-       This Dataset is to construct a dataset from a input shape and label shape.
-       the value range is calculated from: low * stand_normal(0, 1) + high
-
-    Args: sample_size (int): total size of the dummy samples.
-          input_shape (list or tuple): create single or multi input tensors, 
-              tuple reperesent the sample shape of the dataset, eg an image size should be
-              represented as (224, 224, 3), list contains multiple tuple and 
-              represent multi input tensors.
-          label_shape (list or tuple): create single or multi label tensors, 
-              tuple reperesent the label shape of the dataset, eg an label size should be
-              represented as (1, ), list contains multiple tuple and 
-              represent multi label tensors.
-          low (list or float, default=-128.):low out the tensor value range from [0, 1] 
-                                            to [0, low] or [low, 0] if low < 0, if float, 
-                                            will implement all tensors with same low value.  
-          high (list or float, default=127.):high the tensor value by add all tensor element
-                                            value high. If list, length of list should be 
-                                            same with shape list.
-          dtype (list or str, default='float32'):support multi tensor dtype setting. If list,
-                                                length of list should be same with shape list,
-                                                if str, all tensors will use same dtype. dtype
-                                                support 'float32', 'float16', 'uint8', 'int8',
-                                                'int32', 'int64', 'bool'.
-          transform (transform object, default=None): dummy_v2 dataset does not need transform.
-                                                        If transform is not None, it will ignore
-                                                        it.  
-          filter (Filter objects, default=None): filter out examples according to 
-                                                specific conditions
 
+    This Dataset is to construct a dataset from a input shape and label shape.
+    The value range is calculated from: low * stand_normal(0, 1) + high.
     """
+
     def __init__(self, input_shape, label_shape=None, low=-128., high=127., \
                  dtype='float32', transform=None, filter=None):
+        """Initialize `DummyDataset` class.
 
+        Args:
+            sample_size (int): Total size of the dummy samples.
+            input_shape (list or tuple): Create single or multi input tensors,
+                tuple reperesent the sample shape of the dataset, e.g. an image size should be
+                represented as (224, 224, 3), list contains multiple tuple and represent multi input tensors.
+            label_shape (list or tuple): Create single or multi label tensors,
+                tuple reperesent the label shape of the dataset, e.g. an label size should be
+                represented as (1, ), list contains multiple tuple and represent multi label tensors.
+            low (list or float, default=-128.): Low out the tensor value range from [0, 1]
+                to [0, low] or [low, 0] if low < 0. If float, will implement all tensors with same low value.
+            high (list or float, default=127.): High the tensor value by add all tensor element value high.
+                If list, length of list should be same with shape list.
+            dtype (list or str, default='float32'): Support multi tensor dtype setting.
+                If list, length of list should be same with shape list.
+                If str, all tensors will use same dtype.
+                dtype supports 'float32', 'float16', 'uint8', 'int8','int32', 'int64', 'bool'.
+            transform (transform object, default=None): dummy_v2 dataset does not need transform.
+                If transform is not None, it will ignore it.
+            filter (Filter objects, default=None): Filter out examples according to specific conditions.
+        """
         self.dtype_map = {'float32':np.float32, 'float16':np.float16, 'uint8':np.uint8, \
                      'int8':np.int8, 'int32':np.int32, 'int64':np.int64, 'bool':np.bool}
 
@@ -89,14 +87,14 @@ class DummyDataset(IterableDataset):
                 all(isinstance(elem, float) for elem in high),\
                 'high value list length should same with label dim + input_dim'
         else:
-            self.high = (high * np.ones(self.total_dim)).astype(np.float)
+            self.high = (high * np.ones(self.total_dim)).astype(np.float32)
 
         if isinstance(low, list):
             assert len(low) == self.total_dim and \
                 all(isinstance(elem, float) for elem in low), \
                 'low value list length should same with label dim + input_dim'
         else:
-            self.low = (low * np.ones(self.total_dim)).astype(np.float)
+            self.low = (low * np.ones(self.total_dim)).astype(np.float32)
 
         if isinstance(dtype, list):
             assert len(dtype) == self.total_dim and \
@@ -112,6 +110,7 @@ class DummyDataset(IterableDataset):
             self.label_shape = [label_shape]
 
     def __iter__(self):
+        """Yield data in iterative order."""
         while True:
             input_data = []
             for idx in range(0, self.input_dim):
@@ -141,48 +140,44 @@ class DummyDataset(IterableDataset):
                 yield input_data
         
     def __len__(self):
+        """Return the length of dataset."""
         return sys.maxsize
 
-@dataset_registry(dataset_type="sparse_dummy_v2", framework="tensorflow, inteltensorflow, tensorflow_itex, \
+@dataset_registry(dataset_type="sparse_dummy_v2", framework="tensorflow, tensorflow_itex, \
                                                       onnxrt_qlinearops, onnxrt_integerops, \
                                                       pytorch, pytorch_ipex, pytorch_fx, mxnet",
                                                       dataset_format='')
 class SparseDummyDataset(IterableDataset):
     """Dataset used for sparse_dummy_v2 data generation.
-       This Dataset is to construct a dataset from a input shape and label shape.
-       the value range is calculated from: low * stand_normal(0, 1) + high
-
-    Args: sample_size (int): total size of the dummy samples.
-          dense_shape (list or tuple): create single or multi sparse tensors, 
-              tuple reperesent the sample shape of the dataset, eg an image size should be
-              represented as (224, 224, 3), list contains multiple tuple and 
-              represent multi input tensors.
-          label_shape (list or tuple): create single or multi label tensors, 
-              tuple reperesent the label shape of the dataset, eg an label size should be
-              represented as (1, ), list contains multiple tuple and 
-              represent multi label tensors.
-          sparse_ratio (float, default=0.5): the ratio of sparsity, support [0, 1]
-          low (list or float, default=-128.):low out the tensor value range from [0, 1] 
-                                            to [0, low] or [low, 0] if low < 0, if float, 
-                                            will implement all tensors with same low value.  
-          high (list or float, default=127.):high the tensor value by add all tensor element
-                                            value high. If list, length of list should be 
-                                            same with shape list.
-          dtype (list or str, default='float32'):support multi tensor dtype setting. If list,
-                                                length of list should be same with shape list,
-                                                if str, all tensors will use same dtype. dtype
-                                                support 'float32', 'float16', 'uint8', 'int8',
-                                                'int32', 'int64', 'bool'.
-          transform (transform object, default=None): dummy_v2 dataset does not need transform.
-                                                        If transform is not None, it will ignore
-                                                        it.  
-          filter (Filter objects, default=None): filter out examples according to 
-                                                specific conditions
 
+    This Dataset is to construct a dataset from a input shape and label shape.
+    The value range is calculated from: low * stand_normal(0, 1) + high.
     """
+
     def __init__(self, dense_shape, label_shape=None, sparse_ratio=0.5, low=-128., high=127., \
                  dtype='float32', transform=None, filter=None):
+        """Initialize `SparseDummyDataset` class.
 
+        Args:
+            sample_size (int): Total size of the dummy samples.
+            dense_shape (list or tuple): Create single or multi sparse tensors, tuple reperesent
+                the sample shape of the dataset, e.g. an image size should be represented as (224, 224, 3),
+                list contains multiple tuple and represent multi input tensors.
+            label_shape (list or tuple): Create single or multi label tensors, tuple reperesent
+                the label shape of the dataset, e.g. an label size should be represented as (1, ),
+                list contains multiple tuple and represent multi label tensors.
+            sparse_ratio (float, default=0.5): The ratio of sparsity, supports [0, 1].
+            low (list or float, default=-128.): Low out the tensor value range from [0, 1]
+                to [0, low] or [low, 0] if low < 0. If float, will implement all tensors with same low value.
+            high (list or float, default=127.): High the tensor value by add all tensor element value high.
+                If list, length of list should be same with shape list.
+            dtype (list or str, default='float32'): Support multi tensor dtype setting. If list,
+                length of list should be same with shape list. If str, all tensors will use same dtype.
+                dtype supports 'float32', 'float16', 'uint8', 'int8', 'int32', 'int64', 'bool'.
+            transform (transform object, default=None): dummy_v2 dataset does not need transform.
+                If transform is not None, it will ignore it.
+            filter (Filter objects, default=None): Filter out examples according to specific conditions.
+        """
         self.dtype_map = {'float32':np.float32, 'float16':np.float16, 'uint8':np.uint8, \
                      'int8':np.int8, 'int32':np.int32, 'int64':np.int64, 'bool':np.bool}
 
@@ -217,7 +212,7 @@ class SparseDummyDataset(IterableDataset):
                 all(isinstance(elem, float) for elem in sparse_ratio),\
                 'sparse_ratio list length should same with input_dim'
         else:
-            self.sparse_ratio = (sparse_ratio * np.ones(self.input_dim)).astype(np.float)
+            self.sparse_ratio = (sparse_ratio * np.ones(self.input_dim)).astype(np.float32)
         assert all([0 <= i <= 1 for i in self.sparse_ratio]), 'sparse_ratio should be in [0,1]'
 
         if isinstance(high, list):
@@ -225,14 +220,14 @@ class SparseDummyDataset(IterableDataset):
                 all(isinstance(elem, float) for elem in high),\
                 'high value list length should same with label dim + input_dim'
         else:
-            self.high = (high * np.ones(self.total_dim)).astype(np.float)
+            self.high = (high * np.ones(self.total_dim)).astype(np.float32)
 
         if isinstance(low, list):
             assert len(low) == self.total_dim and \
                 all(isinstance(elem, float) for elem in low), \
                 'low value list length should same with label dim + input_dim'
         else:
-            self.low = (low * np.ones(self.total_dim)).astype(np.float)
+            self.low = (low * np.ones(self.total_dim)).astype(np.float32)
 
         if isinstance(dtype, list):
             assert len(dtype) == self.total_dim and \
@@ -242,6 +237,7 @@ class SparseDummyDataset(IterableDataset):
             self.dtype = [self.dtype for i in range(0, self.total_dim)]
 
     def __iter__(self):
+        """Yield data in iterative order."""
         while True:
             input_data = []
             for idx, shape in enumerate(self.dense_shape):
@@ -275,4 +271,5 @@ class SparseDummyDataset(IterableDataset):
             yield input_data
        
     def __len__(self):
+        """Return the length of dataset."""
         return sys.maxsize
