@@ -15,7 +15,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .model import MODELS, BaseModel
+"""Built-in model for multiple framework backends."""
+
+from .model import MODELS, Model
+from .base_model import BaseModel
+
+__all__ = ["MODELS", "Model", "BaseModel"]
 
-__all__ = ["MODELS", "BaseModel"]
 
