@@ -15,7 +15,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .kl_divergence import KL_Divergence
+"""Utils: provide useful methods and auxiliary functionalities."""
+
 from .collect_layer_histogram import LayerHistogramCollector
 from .logger import log, info, debug, warn, warning, error, fatal
 from .options import OPTIONS
