@@ -15,6 +15,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Pytorch utilities."""
+
 from ..adaptor.pytorch import _cfg_to_qconfig, _cfgs_to_fx_cfgs
 from ..adaptor.pytorch import _propagate_qconfig, get_torch_version
 from ..adaptor.pytorch import PyTorch_FXAdaptor
@@ -33,7 +35,7 @@ yaml.SafeLoader.add_constructor('tag:yaml.org,2002:python/tuple',
 
 
 def _set_sub_module_scale_zeropoint(model, tune_cfg, prefix=''):
-    """set activation scale and zero_point for converted sub modules recursively.
+    """Set activation scale and zero_point for converted sub modules recursively.
 
     Args:
         q_model (dir): Int8 model converted from fp32 model. 
@@ -57,7 +59,7 @@ def _set_sub_module_scale_zeropoint(model, tune_cfg, prefix=''):
 
 
 def _set_activation_scale_zeropoint(q_model, tune_cfg):
-    """set activation scale and zero_point for converted model.
+    """Set activation scale and zero_point for converted model.
 
     Args:
         q_model (dir): Int8 model converted from fp32 model. 
@@ -95,7 +97,7 @@ def _set_activation_scale_zeropoint(q_model, tune_cfg):
             _set_sub_module_scale_zeropoint(q_model, tune_cfg)
 
 
-def _load_int8_orchestration(model, tune_cfg, stat_dict, **kwargs):
+def _load_int8_orchestration(model, tune_cfg, stat_dict, example_inputs, **kwargs):
     q_cfgs = torch.quantization.QConfig(
                 activation=torch.quantization.FakeQuantize.with_args(
                         dtype=torch.quint8,
@@ -117,11 +119,7 @@ def _load_int8_orchestration(model, tune_cfg, stat_dict, **kwargs):
         model.train()
         if tune_cfg['sub_module_list'] is None:
             # pylint: disable=E1123
-            if version > Version("1.12.1"):  # pragma: no cover
-                from ..adaptor.pytorch import get_example_inputs
-                example_inputs = get_example_inputs(kwargs["dataloader"] if "dataloader" in
-                                                    kwargs else None)
-
+            if version.release >= Version("1.13.0").release:  # pragma: no cover
                 model = prepare_qat_fx(
                     model,
                     fx_op_cfgs,
@@ -178,12 +176,20 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
             stat_dict = checkpoint_dir
         elif os.path.isfile(checkpoint_dir):
             weights_file = checkpoint_dir
-            stat_dict = torch.load(weights_file)
+            try:
+                stat_dict = torch.jit.load(weights_file)
+                logger.info("torch.jit.load is used to recovery the int8 model quantized by INC IPEX backend")
+            except:
+                stat_dict = torch.load(weights_file)
         elif os.path.isdir(checkpoint_dir):
             try:
                 weights_file = os.path.join(os.path.abspath(os.path.expanduser(checkpoint_dir)),
-                                            'best_model.pt')
-                stat_dict = torch.load(weights_file)
+                                           'best_model.pt')
+                try:
+                    stat_dict = torch.jit.load(weights_file)
+                    logger.info("torch.jit.load is used to recovery the int8 model quantized by INC IPEX backend")
+                except:
+                    stat_dict = torch.load(weights_file)
             except:
                 tune_cfg_file = os.path.join(os.path.abspath(os.path.expanduser(checkpoint_dir)),
                                             'best_configure.yaml')
@@ -196,14 +202,43 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
         else:
             logger.error("Unexpected checkpoint type:{}. \
               Only file dir/path or state_dict is acceptable")
-        assert 'best_configure' in stat_dict, \
-          "No best_configure found in the model file, " \
-          "please use the int8 model file generated by INC"
-        tune_cfg = stat_dict.pop('best_configure')
+
+        if not isinstance(stat_dict, torch.jit._script.RecursiveScriptModule):
+            assert 'best_configure' in stat_dict, \
+                    "No best_configure found in the model file, " \
+                    "please use the int8 model file generated by INC."
+            tune_cfg = stat_dict.pop('best_configure')
     else:
         assert history_cfg is not None, \
           "Need chieckpoint_dir or history_cfg to rebuild int8 model"
         tune_cfg = history_cfg
+        stat_dict = None
+
+    version = get_torch_version()
+    # Get example inputs for ipex and torch>=1.13.
+    example_inputs = None
+    if isinstance(stat_dict, torch.jit._script.RecursiveScriptModule) or \
+      (version.release >= Version("1.13.0").release and \
+      tune_cfg['approach'] != "post_training_dynamic_quant"):
+        if "example_inputs" in kwargs:
+            example_inputs = kwargs["example_inputs"]
+        elif "dataloader" in kwargs:
+            from ..adaptor.pytorch import get_example_inputs
+            example_inputs = get_example_inputs(model, kwargs["dataloader"])
+        else:
+            logger.warning("Please provide the example_inputs or a dataloader" +
+                            " to get example_inputs for quantized model.")
+
+    if isinstance(stat_dict, torch.jit._script.RecursiveScriptModule):
+        q_model = torch.jit.freeze(stat_dict.eval())
+        if isinstance(example_inputs, torch.Tensor):
+            q_model(example_inputs)
+            q_model(example_inputs)
+        else:
+            q_model(*example_inputs)
+            q_model(*example_inputs)
+        logger.info("Finish load the model quantized by INC IPEX backend.")
+        return q_model
 
     try:
         q_model = copy.deepcopy(model)
@@ -213,10 +248,19 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
         q_model = model
 
     if 'is_oneshot' in tune_cfg and tune_cfg['is_oneshot']:
-        return _load_int8_orchestration(q_model, tune_cfg, stat_dict, **kwargs)
+        return _load_int8_orchestration(q_model, tune_cfg, stat_dict, example_inputs, **kwargs)
 
     q_model.eval()
-    version = get_torch_version()
+    approach_quant_mode = None
+    if tune_cfg['approach'] == "post_training_dynamic_quant":
+        approach_quant_mode = 'dynamic'
+    elif tune_cfg['approach'] == "post_training_static_quant":
+        approach_quant_mode = 'static'
+
+    for _, op_cfg in tune_cfg['op'].items():
+        if 'quant_mode' not in op_cfg['activation']:
+            op_cfg['activation']['quant_mode'] = approach_quant_mode
+
     if tune_cfg['approach'] != "post_training_dynamic_quant":
         if version < Version("1.7.0-rc1"):   # pragma: no cover
             q_mapping = tq.default_mappings.DEFAULT_MODULE_MAPPING
@@ -242,15 +286,6 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
         assert version >= Version("1.8.0-rc1"), \
                       "Please use PyTroch 1.8 or higher version with pytorch_fx backend"
         from torch.quantization.quantize_fx import prepare_fx, convert_fx, prepare_qat_fx
-
-        # pragma: no cover
-        if version > Version("1.12.1") and tune_cfg['approach'] != "post_training_dynamic_quant":
-            from ..adaptor.pytorch import get_example_inputs
-            example_inputs = get_example_inputs(kwargs["dataloader"] if "dataloader" in
-                                                kwargs else None)
-        else:
-            example_inputs = None
-
         if kwargs is None:
             kwargs = {}
         prepare_custom_config_dict = kwargs.get(
